@@ -1,16 +1,14 @@
 package Search::Tools::Transliterate;
-use Moo;
-extends 'Search::Tools::Object';
-use Search::Tools::UTF8;
+
+require 5.008;
+use strict;
+use warnings;
 use Carp;
+#use Data::Dumper;      # just for debugging
 use Encode;
-use Encoding::FixLatin qw( fix_latin );
-use Data::Dump qw( dump );
 
-has 'ebit' => ( is => 'rw', default => sub {1} );
-has 'map' => ( is => 'ro' );
+our $VERSION = '0.01';
 
-our $VERSION = '0.999_04';
 
 =pod
 
@@ -21,76 +19,36 @@ Search::Tools::Transliterate - transliterations of UTF-8 chars
 =head1 SYNOPSIS
 
  my $tr = Search::Tools::Transliterate->new();
+ 
  print $tr->convert( 'some string of utf8 chars' );
  
+ 
 =head1 DESCRIPTION
 
 Search::Tools::Transliterate transliterates UTF-8 characters
 to single-byte equivalents. It is based on the transmap project
 by Markus Kuhn http://www.cl.cam.ac.uk/~mgk25/.
 
-B<NOTE:> All the I<is_*> encoding check methods that existed in this class prior
-to version 0.05 were moved to Search::Tools::UTF8 and refactored as functions,
-many using XS for speed improvements.
-
 
 =head1 METHODS
 
 =head2 new
 
-Create new instance. Takes the following optional parameters:
-
-=over
-
-=item map
-
-Customize the character mapping. Should be a hashref. See map() method.
-
-=item ebit
-
-Allow convert() to use full native 8bit characters for transliterating, 
-rather than only 7bit ASCII. The default is true (1). Set to 0 to disable.
-B<NOTE:> This must be set in new(). Changing via the accessor
-after new() will have no effect on map().
-
-=back
-
-=head2 init
-
-Called internally by new().
-
-=head2 map
-
-Access the transliteration character map. Example:
-
- use Search::Tools::Transliterate;
- my $tr = Search::Tools::Transliterate->new;
- $tr->map->{mychar} = 'my transliteration';
- print $tr->convert('mychar');  # prints 'my transliteration'
-
-NOTE: The map() method is an accessor only. You can not pass in a new map.
-
+Create new instance.
 
 =head2 convert( I<text> )
 
-Returns UTF-8 I<text> converted with all single bytes, transliterated according
-to %Map. Will croak if I<text> is not valid UTF-8, so if in doubt, check first with
-is_valid_utf8() in Search::Tools::UTF8.
+Returns I<text> converted with all single bytes, transliterated according
+to %Map.
 
-=head2 convert1252( I<text> )
+=head1 VARIABLES
 
-Returns UTF-8 I<text> converted to all single byte characters,
-transliterated with convert() and the Windows 1252 characters in the range
-B<0x80> and B<0x9f> inclusive. 
+C<%Map> package variable holds all the character mappings. You can alter
+it to taste with:
 
-The 1252 codepoints are converted first to
-their UTF-8 counterparts per 
-http://www.unicode.org/Public/MAPPINGS/VENDORS/MICSFT/WINDOWS/CP1252.TXT
-using Encoding::FixLatin::fix_latin() and then 
-I<text> is run through convert().
-
-Note that I<text> is checked with the looks_like_cp1252() function from
-Search::Tools::UTF8 before calling fix_latin().
+ use Search::Tools::Transliterate;
+ my $tr = Search::Tools::Transliterate->new;
+ $Search::Tools::Transliterate::Map{mychar} = 'my transliteration';
 
 =head1 BUGS
 
@@ -105,84 +63,60 @@ a single space will be used.
 
 =head1 AUTHOR
 
-Peter Karman C<< <karman@cpan.org> >>
-
-Originally based on the HTML::HiLiter regular expression building code, 
-by the same author, copyright 2004 by Cray Inc.
+Peter Karman C<perl@peknet.com>
 
 Thanks to Atomic Learning C<www.atomiclearning.com> 
-for sponsoring the development of some of these modules.
-
-=head1 BUGS
-
-Please report any bugs or feature requests to C<bug-search-tools at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Search-Tools>.  
-I will be notified, and then you'll
-automatically be notified of progress on your bug as I make changes.
-
-=head1 SUPPORT
-
-You can find documentation for this module with the perldoc command.
-
-    perldoc Search::Tools
-
-
-You can also look for information at:
-
-=over 4
-
-=item * RT: CPAN's request tracker
-
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Search-Tools>
-
-=item * AnnoCPAN: Annotated CPAN documentation
-
-L<http://annocpan.org/dist/Search-Tools>
-
-=item * CPAN Ratings
-
-L<http://cpanratings.perl.org/d/Search-Tools>
-
-=item * Search CPAN
-
-L<http://search.cpan.org/dist/Search-Tools/>
-
-=back
+for sponsoring the development of this module.
 
 =head1 COPYRIGHT
 
-Copyright 2006-2010 by Peter Karman.
-
+Copyright 2006 by Peter Karman. 
 This package is free software; you can redistribute it and/or modify it under the 
 same terms as Perl itself.
 
 =head1 SEE ALSO
 
-Search::Tools::UTF8, Unicode::Map, Encode, Test::utf8, Encoding::FixLatin
+Search::Tools, Unicode::Map, Encode
 
 =cut
 
-# must memoize <DATA> the first time since if we call new()
-# more than once, <DATA> has already been iterated over
-# and _init_map() will end up returning empty hash.
-my %MAP;
 
-sub _init_map {
-    my $self = shift;
 
-    return {%MAP} if %MAP;
 
-    while (<DATA>) {
-        chomp;
-        my ( $from, $to ) = (m/^(<U.+?>)\ (.+)$/);
-        my @o = split( /;/, $to );
-        $MAP{ _Utag_to_chr($from) } = _Utag_to_chr( $o[0] );
-    }
-
-    return {%MAP};
+# build map
+our %Map = ();
+while (<DATA>)
+{
+    chomp;
+    my ($from, $to) = (m/^(<U.+?>)\ (.+)$/);
+    $Map{_Utag_to_chr($from)} = _Utag_to_chr($to);
 }
 
-sub _Utag_to_chr {
+# add/override latin1
+for (128 .. 255)
+{
+    $Map{chr($_)} = chr($_);
+}
+
+
+# A Regexp string to match valid UTF8 bytes
+# this info comes from page 78 of "The Unicode Standard 4.0"
+# published by the Unicode Consortium
+our $valid_utf8_regexp = <<EOE;
+        [\x{00}-\x{7f}]
+      | [\x{c2}-\x{df}][\x{80}-\x{bf}]
+      |         \x{e0} [\x{a0}-\x{bf}][\x{80}-\x{bf}]
+      | [\x{e1}-\x{ec}][\x{80}-\x{bf}][\x{80}-\x{bf}]
+      |         \x{ed} [\x{80}-\x{9f}][\x{80}-\x{bf}]
+      | [\x{ee}-\x{ef}][\x{80}-\x{bf}][\x{80}-\x{bf}]
+      |         \x{f0} [\x{90}-\x{bf}][\x{80}-\x{bf}]
+      | [\x{f1}-\x{f3}][\x{80}-\x{bf}][\x{80}-\x{bf}][\x{80}-\x{bf}]
+      |         \x{f4} [\x{80}-\x{8f}][\x{80}-\x{bf}][\x{80}-\x{bf}]
+EOE
+
+
+sub _Utag_to_chr
+{
     my $t = shift;
 
     # cruft
@@ -192,82 +126,62 @@ sub _Utag_to_chr {
     return $t;
 }
 
-sub BUILD {
-    my $self = shift;
-
-    my $map = $self->_init_map;
-
-    # add/override 8bit chars
-    if ( $self->ebit ) {
-        $self->debug and warn "ebit on\n";
-        for ( 128 .. 255 ) {
-            my $c = chr($_);
-            $self->debug and warn "chr $_ -> $c\n";
-            $map->{$c} = $c;
-        }
-    }
-
-    if ( $self->{map} ) {
-        $map->{$_} = $self->{map}->{$_} for keys %{ $self->{map} };
-    }
-    $self->{map} = $map;
+sub new
+{
+    my $class = shift;
+    my $self  = {};
+    bless($self, $class);
+    $self->_init(@_);
+    return $self;
 }
 
-# benchmark shows this is 244% faster than previous version.
-sub convert {
-    my ( $self, $buf ) = @_;
+sub _init
+{
+    my $self  = shift;
+    my %extra = @_;
+    @$self{keys %extra} = values %extra;
+}
+
+sub convert
+{
+    my $self   = shift;
+    my $buf    = shift;
     my $newbuf = '';
-
+    
     # don't bother unless we have non-ascii bytes
-    return $buf if is_ascii($buf);
+    return $buf unless $buf =~ m/[^\x{00}-\x{7f}]/o;
 
-    # make sure we've got valid UTF-8 to start with
-    unless ( is_valid_utf8($buf) ) {
-        my $badbyte = find_bad_utf8($buf);
-        croak "bad UTF-8 byte(s) at $badbyte [ " . dump($buf) . " ]";
-    }
+    Encode::_utf8_off($buf);
 
-    # an alternate algorithm. no idea if it is faster.
-    # it depends on Perl's utf8 char matching (.)
-    # which should work if locale is correct, afaik.
-    my $map = $self->map;
+    while ($buf =~ m/($valid_utf8_regexp)/gox)
+    {
+        my $c            = $1;
+        my $old_char_len = length($c);
+        my $utf          = Encode::decode_utf8($c);
 
-    $self->debug and warn "converting $buf\n";
-    while ( $buf =~ m/(.)/gso ) {
-        my $char = $1;
-        $self->debug and warn "$char\n";
-        if ( is_ascii($char) ) {
-            $self->debug and warn "$char is_ascii\n";
-            $newbuf .= $char;
+        if ($old_char_len == 1)
+        {
+            $newbuf .= $utf;
+            next;
         }
-        elsif ( !exists $map->{$char} ) {
-            $self->debug and warn "$char not in map\n";
+        if (!exists $Map{$utf})
+        {
             $newbuf .= ' ';
-        }
-        else {
-            $self->debug and warn "transliterate $char => $map->{$char}\n";
-            $newbuf .= $map->{$char};
+            next;
         }
 
+        $newbuf .= $Map{$utf};
+
     }
 
+    Encode::_utf8_on($buf);
+
     return $newbuf;
 }
 
-sub convert1252 {
-    my ( $self, $buf ) = @_;
-
-    # don't bother unless we have non-ascii bytes
-    return $buf if is_ascii($buf);
-
-    $self->debug and warn "converting $buf\n";
-    my $newbuf = looks_like_cp1252($buf) ? fix_latin($buf) : $buf;
-    return $self->convert($newbuf);
-}
-
 1;
 
-# map taken directly from
+# map taken directly from 
 # http://www.cl.cam.ac.uk/~mgk25/download/transtab.tar.gz
 # by Markus Kuhn
 
